package ch.epfl.collections;

import static org.junit.Assert.*;

import org.junit.Test;

public abstract class TestMap {
	abstract <K extends Comparable<K>, V> Map<K, V> newInstance();

	@Test
	public void testIsEmpty() {
	    Map<String, String> m = newInstance();
	    assertTrue(m.isEmpty());
	    m.put("hello", "world");
	    assertFalse(m.isEmpty());
	    m.remove("not in the map");
        assertFalse(m.isEmpty());	    
        m.remove("hello");
        assertTrue(m.isEmpty());
	}
	
	@Test
	public void testSize() {
	    Map<String, String> m = newInstance();
	    assertEquals(0, m.size());
	    m.put("one", "un");
	    assertEquals(1, m.size());
        m.put("one", "ein");
        assertEquals(1, m.size());
        m.put("one", "один");
        assertEquals(1, m.size());
        m.remove("two");
        assertEquals(1, m.size());
        m.remove("one");
        assertEquals(0, m.size());
        m.remove("one");
        assertEquals(0, m.size());
	}
	
	@Test
	public void testContainsKey() {
		Map<String, String> m = newInstance();
		assertFalse(m.containsKey("one"));
		m.put("one", "un");
		assertTrue(m.containsKey("one"));
	}

	@Test
	public void testGet() {
		Map<String, String> m = newInstance();
		assertNull(m.get("one"));
		m.put("one", "un");
		assertEquals(m.get("one"), "un");
	}

	@Test
	public void testPut() {
		Map<String, String> m = newInstance();
		m.put("one", "un");
		assertEquals(m.get("one"), "un");
		m.put("one", "uno");
		assertEquals(m.get("one"), "uno");
		m.put("one", "один");
		assertEquals(m.get("one"), "один");
	}

	@Test
	public void testRemove() {
		Map<String, String> m = newInstance();
		m.remove("one");
		assertFalse(m.containsKey("one"));
		m.put("one", "un");
		assertTrue(m.containsKey("one"));
		m.remove("one");
		assertFalse(m.containsKey("one"));
		
		Map<Integer, String> m2 = newInstance();
		for (int i = 0; i < 1000; ++i)
			m2.put(i, String.valueOf(i));
		for (int i = 0; i < 1000; ++i)
			assertTrue(m2.containsKey(i));
		for (int i = 0; i < 500; ++i)
			m2.remove(i);
		for (int i = 0; i < 500; ++i)
			assertFalse(m2.containsKey(i));
		for (int i = 500; i < 1000; ++i)
			assertTrue(m2.containsKey(i));
	}
}
